@@ -1,15 +1,19 @@
 import locale
-
+import os
 import requests
 from flask import Flask, current_app, g
 from flask.ext.principal import identity_loaded
 from flask.ext.sqlalchemy import SQLAlchemy
 from flask.ext.wtf.csrf import CsrfProtect
 
-from .sqlstats import DB_STATS
+
+db = SQLAlchemy()
 
 
-__version__ = '0.4.10'
+from .util import DB_STATS
+
+
+__version__ = '0.5'
 
 
 requests_session = requests.Session()
@@ -19,20 +23,21 @@ requests_session = requests.Session()
 locale.setlocale(locale.LC_ALL, '')
 
 
-db = SQLAlchemy()
-
-
 csrf = CsrfProtect()
 
 
 # Ensure models are declared
 from . import models
-from .auth import models
+from .auth import models as auth_models
 
 
-def create_app(**kwargs):
+def create_app(config=None, **kwargs):
     app = Flask('evesrp', **kwargs)
     app.config.from_object('evesrp.default_config')
+    if config is not None:
+        app.config.from_pyfile(config)
+    if app.config['SECRET_KEY'] is None and 'SECRET_KEY' in os.environ:
+        app.config['SECRET_KEY'] = os.environ['SECRET_KEY']
 
     # Register SQLAlchemy monitoring before the DB is connected
     app.before_request(sqlalchemy_before)
@@ -42,8 +47,9 @@ def create_app(**kwargs):
     from .views.login import login_manager
     login_manager.init_app(app)
 
-    from .auth import principal
+    from .auth.permissions import principal, load_user_permissions
     principal.init_app(app)
+    identity_loaded.connect(load_user_permissions, app)
 
     before_csrf = list(app.before_request_funcs[None])
     csrf.init_app(app)
@@ -51,25 +57,40 @@ def create_app(**kwargs):
     # is the template function.
     app.before_request_funcs[None] = before_csrf
 
-    from .views import index, divisions, login, requests, api
+    from .views import index, error_page, divisions, login, requests, api
     app.add_url_rule(rule='/', view_func=index)
+    for error_code in (400, 403, 404, 500):
+        app.register_error_handler(error_code, error_page)
     app.register_blueprint(divisions.blueprint, url_prefix='/divisions')
     app.register_blueprint(login.blueprint)
     app.register_blueprint(requests.blueprint)
     app.register_blueprint(api.api, url_prefix='/api')
     app.register_blueprint(api.filters, url_prefix='/api/filter')
 
+    from .views import request_count
+    app.add_template_global(request_count)
+
     from .json import SRPEncoder
     app.json_encoder=SRPEncoder
 
-    from .auth import load_user_permissions
-    identity_loaded.connect(load_user_permissions, app)
-
     app.before_first_request(_copy_config_to_authmethods)
     app.before_first_request(_config_requests_session)
     app.before_first_request(_config_killmails)
     app.before_first_request(_copy_url_converter_config)
 
+    # Configure the Jinja context
+    # Inject variables into the context
+    from .auth import PermissionType
+    @app.context_processor
+    def inject_enums():
+        return {
+            'ActionType': models.ActionType,
+            'PermissionType': PermissionType,
+        }
+    # Auto-trim whitespace
+    app.jinja_env.trim_blocks = True
+    app.jinja_env.lstrip_blocks = True
+
     return app
 
 
